@@ -27,6 +27,9 @@ struct pwquality_settings {
         int dict_check;
         int user_check;
         int enforcing;
+        int retry_times;
+        int enforce_for_root;
+        int local_users_only;
         char *bad_words;
         char *dict_path;
 };
@@ -46,6 +49,9 @@ struct setting_mapping {
 #define PWQ_DEFAULT_DICT_CHECK   1
 #define PWQ_DEFAULT_USER_CHECK   1
 #define PWQ_DEFAULT_ENFORCING    1
+#define PWQ_DEFAULT_RETRY_TIMES  1
+#define PWQ_DEFAULT_ENFORCE_ROOT 0
+#define PWQ_DEFAULT_LOCAL_USERS  0
 
 #define PWQ_TYPE_INT             1
 #define PWQ_TYPE_STR             2
