@@ -10,6 +10,8 @@
 #ifndef PWQUALITY_H
 #define PWQUALITY_H
 
+#include <sys/types.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -30,6 +32,9 @@ extern "C" {
 #define PWQ_SETTING_DICT_CHECK      15
 #define PWQ_SETTING_USER_CHECK      16
 #define PWQ_SETTING_ENFORCING       17
+#define PWQ_SETTING_RETRY_TIMES     18
+#define PWQ_SETTING_ENFORCE_ROOT    19
+#define PWQ_SETTING_LOCAL_USERS     20
 
 #define PWQ_MAX_ENTROPY_BITS       256
 #define PWQ_MIN_ENTROPY_BITS       56
